@@ -6,14 +6,12 @@
  */
 
 import path from 'path';
-import {promisify} from 'util';
 
-import {fs} from 'mz';
-import mkdirp from 'mkdirp';
+import fs from 'fs-extra';
+import asyncMkdirp from 'mkdirp';
 import {
-  LaunchedChrome,
+  Launcher as ChromeLauncher,
   launch as defaultChromiumLaunch,
-  default as ChromeLauncher,
 } from 'chrome-launcher';
 import WebSocket from 'ws';
 
@@ -23,6 +21,8 @@ import type {
   ExtensionRunnerParams,
   ExtensionRunnerReloadResult,
 } from './base';
+import isDirectory from '../util/is-directory';
+import fileExists from '../util/file-exists';
 
 type ChromiumSpecificRunnerParams = {|
    chromiumBinary?: string,
@@ -38,8 +38,6 @@ export type ChromiumExtensionRunnerParams = {|
 
 const log = createLogger(__filename);
 
-const asyncMkdirp = promisify(mkdirp);
-
 const EXCLUDED_CHROME_FLAGS = [
   '--disable-extensions',
   '--mute-audio',
@@ -54,7 +52,7 @@ export const DEFAULT_CHROME_FLAGS = ChromeLauncher.defaultFlags()
 export class ChromiumExtensionRunner {
   cleanupCallbacks: Set<Function>;
   params: ChromiumExtensionRunnerParams;
-  chromiumInstance: LaunchedChrome;
+  chromiumInstance: ChromeLauncher;
   chromiumLaunch: typeof defaultChromiumLaunch;
   reloadManagerExtension: string;
   wss: WebSocket.Server;
@@ -85,6 +83,52 @@ export class ChromiumExtensionRunner {
     await this._promiseSetupDone;
   }
 
+  static async isUserDataDir(dirPath: string): Promise<boolean> {
+    const localStatePath = path.join(dirPath, 'Local State');
+    const defaultPath = path.join(dirPath, 'Default');
+    // Local State and Default are typical for the user-data-dir
+    return await fileExists(localStatePath)
+      && await isDirectory(defaultPath);
+  }
+
+  static async isProfileDir(dirPath: string): Promise<boolean> {
+    const securePreferencesPath = path.join(
+      dirPath, 'Secure Preferences');
+    //Secure Preferences is typical for a profile dir inside a user data dir
+    return await fileExists(securePreferencesPath);
+  }
+
+  static async getProfilePaths(chromiumProfile: ?string): Promise<{
+    userDataDir: ?string,
+    profileDirName: ?string
+  }> {
+    if (!chromiumProfile) {
+      return {
+        userDataDir: null,
+        profileDirName: null,
+      };
+    }
+
+    const isProfileDirAndNotUserData =
+      await ChromiumExtensionRunner.isProfileDir(chromiumProfile)
+      && !await ChromiumExtensionRunner.isUserDataDir(chromiumProfile);
+
+    if (isProfileDirAndNotUserData) {
+      const {dir: userDataDir, base: profileDirName} =
+        path.parse(chromiumProfile);
+      return {
+        userDataDir,
+        profileDirName,
+      };
+    }
+
+    return {
+      userDataDir: chromiumProfile,
+      profileDirName: null,
+    };
+
+  }
+
   /**
    * Setup the Chromium Profile and run a Chromium instance.
    */
@@ -130,8 +174,43 @@ export class ChromiumExtensionRunner {
       chromeFlags.push(...this.params.args);
     }
 
-    if (this.params.chromiumProfile) {
-      chromeFlags.push(`--user-data-dir=${this.params.chromiumProfile}`);
+    // eslint-disable-next-line prefer-const
+    let {userDataDir, profileDirName} =
+      await ChromiumExtensionRunner.getProfilePaths(
+        this.params.chromiumProfile);
+
+    if (userDataDir && this.params.keepProfileChanges) {
+      if (profileDirName
+        && !await ChromiumExtensionRunner.isUserDataDir(userDataDir)) {
+        throw new Error('The profile you provided is not in a ' +
+          'user-data-dir. The changes cannot be kept. Please either ' +
+          'remove --keep-profile-changes or use a profile in a ' +
+          'user-data-dir directory');
+      }
+    } else if (!this.params.keepProfileChanges) {
+      // the user provided an existing profile directory but doesn't want
+      // the changes to be kept. we copy this directory to a temporary
+      // user data dir.
+      const tmpDir = new TempDir();
+      await tmpDir.create();
+      const tmpDirPath = tmpDir.path();
+
+      if (userDataDir && profileDirName) {
+        // copy profile dir to this temp user data dir.
+        await fs.copy(path.join(
+          userDataDir,
+          profileDirName), path.join(
+          tmpDirPath,
+          profileDirName),
+        );
+      } else if (userDataDir) {
+        await fs.copy(userDataDir, tmpDirPath);
+      }
+      userDataDir = tmpDirPath;
+    }
+
+    if (profileDirName) {
+      chromeFlags.push(`--profile-directory=${profileDirName}`);
     }
 
     let startingUrl;
@@ -147,6 +226,7 @@ export class ChromiumExtensionRunner {
       chromePath: chromiumBinary,
       chromeFlags,
       startingUrl,
+      userDataDir,
       // Ignore default flags to keep the extension enabled.
       ignoreDefaultFlags: true,
     });
