@@ -5,7 +5,6 @@ import {readFileSync} from 'fs';
 
 import camelCase from 'camelcase';
 import decamelize from 'decamelize';
-import git from 'git-rev-sync';
 import yargs from 'yargs';
 import { Parser as yargsParser } from 'yargs/yargs';
 
@@ -153,7 +152,20 @@ export class Program {
   // Retrieve the yargs argv object and apply any further fix needed
   // on the output of the yargs options parsing.
   getArguments(): Object {
+    // To support looking up required parameters via config files, we need to
+    // temporarily disable the requiredArguments validation. Otherwise yargs
+    // would exit early. Validation is enforced by the checkRequiredArguments()
+    // method, after reading configuration files.
+    //
+    // This is an undocumented internal API of yargs! Unit tests to avoid
+    // regressions are located at: tests/functional/test.cli.sign.js
+    //
+    // Replace hack if possible:  https://github.com/mozilla/web-ext/issues/1930
+    const validationInstance = this.yargs.getValidationInstance();
+    const { requiredArguments } = validationInstance;
+    validationInstance.requiredArguments = () => {};
     const argv = this.yargs.argv;
+    validationInstance.requiredArguments = requiredArguments;
 
     // Yargs boolean options doesn't define the no* counterpart
     // with negate-boolean on Yargs 15. Define as expected by the
@@ -178,6 +190,15 @@ export class Program {
     return argv;
   }
 
+  // getArguments() disables validation of required parameters, to allow us to
+  // read parameters from config files first. Before the program continues, it
+  // must call checkRequiredArguments() to ensure that required parameters are
+  // defined (in the CLI or in a config file).
+  checkRequiredArguments(adjustedArgv: Object): void {
+    const validationInstance = this.yargs.getValidationInstance();
+    validationInstance.requiredArguments(adjustedArgv);
+  }
+
   // Remove WEB_EXT_* environment vars that are not a global cli options
   // or an option supported by the current command (See #793).
   cleanupProcessEnvConfigs(systemProcess: typeof process) {
@@ -287,6 +308,8 @@ export class Program {
         this.enableVerboseMode(logStream, version);
       }
 
+      this.checkRequiredArguments(adjustedArgv);
+
       await runCommand(adjustedArgv, {shouldExitProgram});
 
     } catch (error) {
@@ -329,6 +352,11 @@ export function defaultVersionGetter(
     return JSON.parse(packageData).version;
   } else {
     log.debug('Getting version from the git revision');
+    // This branch is only reached during development.
+    // git-rev-sync is in devDependencies, and lazily imported using require.
+    // This also avoids logspam from https://github.com/mozilla/web-ext/issues/1916
+    // eslint-disable-next-line import/no-extraneous-dependencies
+    const git = require('git-rev-sync');
     return `${git.branch(absolutePackageDir)}-${git.long(absolutePackageDir)}`;
   }
 }
@@ -342,6 +370,15 @@ type MainParams = {
   runOptions?: Object,
 }
 
+export function throwUsageErrorIfArray(errorMessage: string): any {
+  return (value: any): any => {
+    if (Array.isArray(value)) {
+      throw new UsageError(errorMessage);
+    }
+    return value;
+  };
+}
+
 export function main(
   absolutePackageDir: string,
   {
@@ -429,6 +466,18 @@ Example: $0 --help run.
       default: true,
       type: 'boolean',
     },
+    'filename': {
+      alias: 'n',
+      describe: 'Name of the created extension package file.',
+      default: undefined,
+      normalize: false,
+      demandOption: false,
+      requiresArg: true,
+      type: 'string',
+      coerce: throwUsageErrorIfArray(
+        'Multiple --filename/-n option are not allowed'
+      ),
+    },
   });
 
   program
@@ -618,6 +667,11 @@ Example: $0 --help run.
         type: 'number',
         requiresArg: true,
       },
+      'adb-remove-old-artifacts': {
+        describe: 'Remove old artifacts directories from the adb device',
+        demandOption: false,
+        type: 'boolean',
+      },
       'firefox-apk': {
         describe: (
           'Run a specific Firefox for Android APK. ' +
