@@ -8,6 +8,9 @@ import {
 } from '../errors';
 import {createLogger} from '../util/logger';
 
+export const DEVICE_DIR_BASE = '/sdcard/';
+export const ARTIFACTS_DIR_PREFIX = 'web-ext-artifacts-';
+
 const log = createLogger(__filename);
 
 export type ADBUtilsParams = {|
@@ -120,7 +123,8 @@ export default class ADBUtils {
           line.startsWith('org.mozilla.fennec') ||
           line.startsWith('org.mozilla.fenix') ||
           line.startsWith('org.mozilla.geckoview') ||
-          line.startsWith('org.mozilla.firefox')
+          line.startsWith('org.mozilla.firefox') ||
+          line.startsWith('org.mozilla.reference.browser')
         );
       });
   }
@@ -194,7 +198,7 @@ export default class ADBUtils {
       return artifactsDir;
     }
 
-    artifactsDir = `/sdcard/web-ext-artifacts-${Date.now()}`;
+    artifactsDir = `${DEVICE_DIR_BASE}${ARTIFACTS_DIR_PREFIX}${Date.now()}`;
 
     const testDirOut = (await this.runShellCommand(
       deviceId, `test -d ${artifactsDir} ; echo $?`
@@ -214,6 +218,44 @@ export default class ADBUtils {
     return artifactsDir;
   }
 
+  async detectOrRemoveOldArtifacts(
+    deviceId: string, removeArtifactDirs?: boolean = false
+  ): Promise<boolean> {
+    const {adbClient} = this;
+
+    log.debug('Checking adb device for existing web-ext artifacts dirs');
+
+    return wrapADBCall(async () => {
+      const files = await adbClient.readdir(deviceId, DEVICE_DIR_BASE);
+      let found = false;
+
+      for (const file of files) {
+        if (!file.isDirectory() ||
+            !file.name.startsWith(ARTIFACTS_DIR_PREFIX)) {
+          continue;
+        }
+
+        // Return earlier if we only need to warn the user that some
+        // existing artifacts dirs have been found on the adb device.
+        if (!removeArtifactDirs) {
+          return true;
+        }
+
+        found = true;
+
+        const artifactsDir = `${DEVICE_DIR_BASE}${file.name}`;
+
+        log.debug(
+          `Removing artifacts directory ${artifactsDir} from device ${deviceId}`
+        );
+
+        await this.runShellCommand(deviceId, ['rm', '-rf', artifactsDir]);
+      }
+
+      return found;
+    });
+  }
+
   async clearArtifactsDir(deviceId: string): Promise<void> {
     const artifactsDir = this.artifactsDirMap.get(deviceId);
 
@@ -228,9 +270,7 @@ export default class ADBUtils {
       `Removing ${artifactsDir} artifacts directory on ${deviceId} device`
     );
 
-    await this.runShellCommand(deviceId, [
-      'rm', '-rf', artifactsDir,
-    ]);
+    await this.runShellCommand(deviceId, ['rm', '-rf', artifactsDir]);
   }
 
   async pushFile(
@@ -270,8 +310,14 @@ export default class ADBUtils {
       value: `-profile ${deviceProfileDir}`,
     }];
 
-    const component = apkComponent ?
-      `${apk}/.${apkComponent}` : `${apk}/.App`;
+    if (!apkComponent) {
+      apkComponent = '.App';
+    } else if (!apkComponent.includes('.')) {
+      apkComponent = `.${apkComponent}`;
+    }
+    // if `apkComponent` starts with a '.', then adb will expand
+    // the following to: `${apk}/${apk}.${apkComponent}`
+    const component = `${apk}/${apkComponent}`;
 
     await wrapADBCall(async () => {
       await adbClient.startActivity(deviceId, {
